import requests
import re
import json
import base64
import hashlib
from rogue_tools import path_tool

class MsgRobot():
    def __init__(self,webhook_list=[],webhook=None) -> None:
        self.webhook_list = webhook_list
        if webhook:
            self.webhook_list.append(webhook)

    def is_feishu(self,webhook_url):
        group = re.search(r'(feishu)',webhook_url, re.I)
        rs = group.group() if group else None
        return True if rs else False

    def is_weixin(self,webhook_url):
        group = re.search(r'(weixin)',webhook_url, re.I)
        rs = group.group() if group else None
        return True if rs else False
        
    def push_all(self,msg_or_pic:str):
        if len(msg_or_pic)>1024:
            msg_or_pic = msg_or_pic[-1024:]
        if path_tool.is_exists(msg_or_pic) and msg_or_pic[-4:] in ('.jpg','.png'):
            self.push_pic(msg_or_pic) 
        else:
            self.push_text(msg_or_pic)

    def push_pic(self,msg):
        for webhook in self.webhook_list:
            if self.is_feishu(webhook):
                self.fs_push_pic(webhook,msg)
            elif self.is_weixin(webhook):
                self.wx_push_pic(webhook,msg)

    def push_text(self,msg):
        for webhook in self.webhook_list:
            if self.is_feishu(webhook):
                self.fs_push_text(webhook,msg)
            elif self.is_weixin(webhook):
                self.wx_push_text(webhook,msg)

    def _wx_push_main(self,webhook,data):
        # 企业微信机器人的 webhook
        # 开发文档 https://work.weixin.qq.com/api/doc#90000/90136/91770
        #webhook = f"https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key={key}"
        headers = {'content-type': 'application/json'}  # 请求头
        r = requests.post(webhook, headers=headers, data=json.dumps(data))
        r.encoding = 'utf-8'
        return r.text

    def _fs_push_main(self,webhook,data):
        headers = {'Content-Type': 'application/json'}
        response = requests.post(webhook, headers=headers, data=json.dumps(data))
        if response.status_code == 200:
            return '消息发送成功'
        else:
            return f'消息发送失败:{response.status_code}'

    def wx_push_pic(self,webhook,pic_path):
        pic_obj =None
        with open(pic_path,'rb') as pic:
            pic_obj = pic.read()
        pic_64  =  base64.b64encode(pic_obj)
        pic_str = str(pic_64,'utf-8')
        pic_md5 = hashlib.md5(pic_obj).hexdigest()
        # 发送图片
        webhook_data = {"msgtype": "image","image": {"base64": pic_str,"md5": pic_md5}}
        # 企业微信机器人发送
        self._wx_push_main(webhook, webhook_data)

    def fs_push_pic(self,webhook,msg):
        self.fs_push_text(webhook,'暂不支持发送飞书图片,假装这里有个超酷炫的图片吧')

    def wx_push_text(self,webhook,msg:str):
        webhook_data = {"msgtype": "text","text": {"content": msg}}
        if msg.startswith('markdown'):
            temp = msg.split('_',2)
            webhook_data = {"msgtype": "markdown","markdown": {"content": f'{temp[1]}\n<font color=\"comment\">{temp[2]}</font>'}}
        return self._wx_push_main(webhook,webhook_data)

    def fs_push_text(self,webhook,msg):
        webhook_data = {'msg_type': 'text', 'content': {'text': msg}}
        return self._fs_push_main(webhook,webhook_data)
    




def select_filter_list(src_list,include_list=[],exclude_list=[]):
    '''
    检查src_list中的元素,是否满足过滤器的过滤条件
    '''
    if src_list==[]:
        return src_list
    if include_list==[] and exclude_list==[]:
        return src_list

    rs_list = []
    for src_str in src_list:
        if is_in_list(src_str,exclude_list):
            continue
        if len(include_list) == 0 or is_in_list(src_str,include_list):
            rs_list.append(src_str)
            continue
    return rs_list
    
def is_in_list(src_str,find_list):
    '''
    遍历检查字符串是否包含find_list中的元素(str类型)
    '''
    for find_str in find_list:
        if src_str.find(find_str)>-1:
            return True
    return False

def get_diff(list1,list2)->list:
    '''
    两个列表之间的差异
    '''
    rs_list =[]
    rs_list.extend(list(set(list1).difference(set(list2))))
    rs_list.extend(list(set(list2).difference(set(list1))))
    return rs_list

def get_same(list1,list2)->list:
    '''
    同时存在于list1和list2
    '''
    rs_list = list(set(list1).intersection(set(list2)))
    return rs_list


import sys
import os
import time
import shutil
import re
import subprocess

win_install_dict = {
'PyQt5':None,
'loguru':None,
#'airtest':None,
'poco':None,
'pocoui':None,
'requests':None,
'openpyxl':None,
#'jinja2':'3.0.1',
'rogue-tools':None,
}
linux_install_dict = {
'loguru':None,
'poco':None,
'pocoui':None,
'requests':None,
'openpyxl':None,
'rogue-tools':None,
}




class Singleton(type):
    _instances = {}
    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super().__call__(*args, **kwargs)
        return cls._instances[cls]

class InstallModule(metaclass=Singleton):
	def __init__(self,python_exe,pip_exe) -> None:
		self.python_exe = python_exe
		self.pip_exe = pip_exe
		self.mods_version_dic={}
		self.last_update_result={}

	def load_mods_version(self):
		'''获得当前所有第三方库的版本'''
		result = subprocess.run(f'{self.pip_exe} list', shell=True, capture_output=True, text=True)
		output_lines = result.stdout.strip().split('\n')[2:]  # 跳过前两行标题
		for line in output_lines:
			package_name,package_version = line.split()
			self.mods_version_dic[package_name] = package_version


	def update_mod(self,mod_name,mod_version=None):
		'''安装单mod个'''
		uninstall_cmd = f'{self.pip_exe} uninstall -y {mod_name}'
		subprocess.run(uninstall_cmd, shell=True)

		if mod_version:
			install_cmd = f'{self.pip_exe} install --no-cache-dir {mod_name}=={mod_version}'
		else:
			install_cmd = f'{self.pip_exe} install --no-cache-dir {mod_name}'
		subprocess.run(install_cmd, shell=True)

		return True

	def update_mods(self,mods:dict):
		self.last_update_result=mods.copy()
		self.load_mods_version()
		for mod_name,version in mods.items():
			now_version=self.mods_version_dic.get(mod_name,0)
			if now_version!=version:
				try:
					self.update_mod(mod_name,version)
					self.last_update_result[mod_name]='已经更新'
				except BaseException:
					self.last_update_result[mod_name]='更新失败'
			else:
				self.last_update_result[mod_name]='无需更新'
		return self.last_update_result

	def show_result(self):
		for key,value in self.last_update_result.items():
			print(key,value)

	def cover_mod(self):
		site_packages_path = get_site_packages_path()

	def get_default_python(self):
		'''获得python安装目录'''
		f=os.popen(f'{self.python_exe} -0p')
		cmd_rs=f.read().splitlines()
		for line_str in cmd_rs:
			info_list=[]
			if len(line_str)>0:
				for info in line_str.split(' '):
					if len(info)>0:
						info_list.append(info)
			if info_list:
				python_version = info_list[0]
				python_path    = info_list[1]            
				return python_version,python_path

def get_site_packages_path():
	'''获取第三方库的路径'''
	for path in sys.path:
		if r'\\Lib\site-packages\\' in path:
			return path










import os
import traceback
import math

import openpyxl
from openpyxl.worksheet.worksheet import Worksheet
from rogue_tools import path_tool



def standard_list(src_list,list_len,standard_obj=None):
    '''
    长的截断,短的填充standard_obj
    '''
    if len(src_list)<list_len:
        for i in range(0,list_len-len(src_list)):
            src_list.append(standard_obj)
    
    return src_list[0:list_len]


def is_in_filter(src_str,include_list=[],exclude_list=[]):
    '''
    检查src_str是否符合筛选条件(str类型)
    '''
    if include_list==[] and exclude_list==[]:
        return True

    for find_str in exclude_list:
        if src_str.find(find_str)>-1:
            return False
    for find_str in include_list:
        if src_str.find(find_str)>-1:
            return True
    return False

def calc_26_to_10(a_z:str):
    rs=0
    a_z = a_z.lower()
    sq = len(a_z)-1
    for word in a_z:
        number = (ord(word)-ord('a')+1)*math.pow(26,sq)
        sq=sq-1
        rs = rs+number
    return int(rs)



def cut_list_tail(src_list,list_obj):
    for i in range(0,len(src_list)):
        index = len(src_list)-i-1
        if src_list[index]!=list_obj:
            return src_list[0:index+1]
    return []


class MySheet():
    '''
    仅保留有效数据组成的矩阵,获取行或列的时候,请从第一行数据开始,从0开始
    处理表头,用专门的方法
    '''
    def __init__(self,sheet:Worksheet) -> None:
        if sheet == None:
            return
        #self.start_line         = 0
        self.error_list         = []
        self.row_list           = [] # 是一个规整的二维矩阵
        self.column_list        = [] # 是一个规整的二维矩阵
        self.key_list           = []
        self.name               = sheet.title
        if sheet.max_row==0 or sheet.max_column==0:
            return
        
        # 加载数据
        for row in sheet.rows:
            row_value = []
            for cell in row:
                value = cell.value if cell.value or cell.value == 0 else ''
                row_value.append(value)
            self.row_list.append(row_value)
        
        for column in sheet.columns:
            column_value = []
            for cell in column:
                value = cell.value if cell.value or cell.value == 0 else ''
                column_value.append(value)
            self.column_list.append(column_value)
        # 标记序列的行
        self.key_list = cut_list_tail(self.row_list[0],'')
        self.index_list = cut_list_tail(self.column_list[0],'')
        # 获得最大行列
        self.max_row = len(self.index_list)
        self.max_column= len(self.key_list)
        # 重新规整数据
        del self.row_list[self.max_row:]
        for i in range(0,len(self.row_list)-1):
            del self.row_list[i][self.max_column:]
        
        #print(f'init {self.name}')

    def get_value(self,x,y):
        '''
        通过表内坐标获得value,从0,0开始
        '''
        return self.row_list[x][y]
    def get_all_rows(self):
        return self.row_list
    def get_row_list(self,row):
        '''
        取得某一行，从0开始
        '''
        if row >= self.max_row or row < 0:
            return None
        else:
            return self.row_list[row]

    def get_column_list(self,key):
        '''
        取得某一列
        '''

        if type(key)==int:
            if key >= self.max_column or key < 0:
                return None
            else:
                return self.column_list[key]
        else:
            return self.get_column_list(calc_26_to_10(key)-1)



class MyExcel():
    def __init__(self,excel_file_path) -> None:
        self.excel_file_path             = excel_file_path
        self._work_book                  = None
        self.sheet_dic                   = {}
        self._sheet_name_list            = []
        self.is_read                     = False
        self.read_file()

    def read_file(self):
        if os.path.exists(self.excel_file_path):
            try:
                self._work_book:openpyxl.Workbook   = openpyxl.load_workbook(self.excel_file_path) # 打开Excel文件
                self._sheet_name_list               = self._work_book.sheetnames
                for sheet_name in self._sheet_name_list:
                    self.sheet_dic[sheet_name] = MySheet(self._work_book[sheet_name])
                self.is_read = True
            except Exception:
                traceback.print_exc()
                print(f'加载Excel文件失败:{sheet_name} {self.excel_file_path}')
            finally:
                self._work_book.close()

    def write_excel(self,other_file=None,is_backup=True):
        '''
        修改这个Myexcel
        '''
        if os.path.exists(self.excel_file_path):
            self._work_book = openpyxl.load_workbook(self.excel_file_path)
        else:
            self._work_book = openpyxl.Workbook()

        try:
            for sheet_name in self.sheet_dic:
                sheet_row_value = self.sheet_dic[sheet_name].row_list # 按行分配的单元格列表
                if not sheet_row_value:
                    sheet_row_value=[['']]

                sheet = self._work_book[sheet_name]
                for row_index in range(0,len(sheet_row_value)):
                    row_value = sheet_row_value[row_index]
                    for col_index in range(0,len(row_value)):
                        sheet.cell(row_index+1,col_index+1,f'{row_value[col_index]}')

            save_path = self.excel_file_path
            if other_file:
                save_path = other_file
            if is_backup:
                path_tool.copy(self.excel_file_path,self.excel_file_path+'.bak')
            self._work_book.save(save_path)
            
        except Exception:
            traceback.print_exc()
        finally:
            self._work_book.close()
            
            

    
    def get_sheet_name_list(self):
        return self._sheet_name_list

    def get_sheet(self,sheet_name) -> MySheet:
        return self.sheet_dic.get(sheet_name,None)
    
    def add_sheet(self,sheet_name)-> MySheet:
        self._sheet_name_list.append(sheet_name)
        self.sheet_dic[sheet_name]=MySheet(sheet_name)
        return self.sheet_dic[sheet_name]
def calc_26_to_10(a_z:str):
    rs=0
    a_z = a_z.lower()
    sq = len(a_z)-1
    for word in a_z:
        number = (ord(word)-ord('a')+1)*math.pow(26,sq)
        sq=sq-1
        rs = rs+number
    return int(rs)




import time
def time_stamp():
	'''
	超精准时间戳
	'''
	return int(round(time.time() * 1000000))

def check_time_stamp(ts=0,print_func=None):
	'''
	配合time_stamp使用,debug时候用的时间戳
	'''
	now = time_stamp()
	if ts>0:
		if print_func:
			print_func(f'{now - ts}')
		else:
			print(f'{now - ts}')
	return now

def time_stamp_s():
	'''
	秒级时间戳
	'''
	return int(round(time.time()))

def time_stamp_ms():
	'''
	超精准时间戳
	'''
	return int(round(time.time() * 1000))


def time_for_read(time_stamp = None,delay_stamp_ms=0):
	'''
	提供一种可读性好,且较为精准的时间戳
	精确到ms
	delay_stamp_ms为延迟后X毫秒后的时间戳
	'''
	if time_stamp == None:
		time_stamp = time.time()
	x = str(int(time_stamp * 1000) % 1000).rjust(3,'0')
	return time.strftime(f"%H-%M-%S-{x}", time.localtime(time_stamp+delay_stamp_ms/1000))
def time_for_read_away(t1,t2):
	'''
	辅助time_for_read进行差值计算,返回是ms
	'''
	tp1 = t1.split('-')
	tp2 = t2.split('-')
	return (int(tp2[0])-int(tp1[0]))*3600000+(int(tp2[1])-int(tp1[1]))*60000+(int(tp2[2])-int(tp1[2]))*1000+(int(tp2[3])-int(tp1[3]))
	


def get_now_ms(time_stamp = None,delay_stamp_ms=0):
	if time_stamp == None:
		time_stamp = time.time()
	x = str(int(time_stamp * 1000) % 1000).rjust(3,'0')
	return time.strftime(f"%Y-%m-%d_%H-%M-%S-{x}", time.localtime(time_stamp+delay_stamp_ms/1000))

def get_now_time():
	return time.strftime("%Y-%m-%d_%H-%M-%S", time.localtime())
def get_now_date():
	return time.strftime("%Y-%m-%d", time.localtime())

def get_next_date(days=1):
	'''
	之前的时间，默认一天后,也可以负数呀
	'''
	return time.strftime("%Y-%m-%d", time.localtime(time.time()-86400*days))

def get_next_hour(hour=1):
	'''
	之前的时间，默认一小时前,也可以负数呀，还可以小数呀
	'''
	return time.strftime("%Y-%m-%d_%H-%M-%S", time.localtime(time.time()-3600*hour))

def countdown(sleep_time):
	print(f'countdown:{sleep_time}/{sleep_time}')
	for i in range(1,sleep_time+1):
		time.sleep(1)
		print(f'countdown:{sleep_time-i}/{sleep_time}')




import os
import stat
import sys
import re
import shutil
import time

import traceback
from rogue_tools import path_tool,file_tool,thread_tool

pool = None
def init_pool():
    '''
    用来加快删除文件夹操作的进程
    '''
    global pool
    if pool==None:
        pool = thread_tool.ThreadPool()



'''        
#####################################################################
#                             简易方法                               #    
#####################################################################   
'''
def copy_(src,tar):


    #print(f'copy:{src} -->> {tar}')
    src_status =  get_path_status(src)
    tar_status =  get_path_status(tar)
    if src_status in (3,4):
        print(f'can not find src :\n{src}')
        return False
    if src_status==1 and tar_status in (1,3):           # 文件到文件
        shutil.copyfile(src, tar)
        return True

    elif src_status==1 and tar_status in (2,4):       # 文件到目录
        make_folder(tar)
        base_name = get_file_base_name(src)
        tar = join_path(tar,base_name)
        copy_(src, tar)
        return True

    if src_status==2 and tar_status in (1,3):       # 目录到文件
        # 尝试将一个文件夹复制为文件
        print(f'can not copy folder to file.\n[{src}]\n-->>[{tar}]')
        return False
    if src_status==2 and tar_status in (2,4):   # 目录到目录
        make_folder(tar)
        get_all_permission(tar)
        sub_file_list = get_sub_files(src,full_path=False)
        for sub_file in sub_file_list:
            copy_(join_path(src,sub_file),tar)
        return True
        
def copy(src, tar,is_cover=False):
    '''
    通用的复制文件或者文件夹
    is_cover = True 可以不用删除旧文件夹,直接覆盖过去
    仅支持本地copy
    '''
    #print(f'copy:{src} -->> {tar}')
    if type(src) in [tuple,list]:
        for obj in src:
            copy(obj,tar,is_cover)
        return
    # copy文件
    if os.path.isfile(src):
        #  复制到目录
        if os.path.isdir(tar):
            base_name = get_file_base_name(src)
            return shutil.copyfile(src, join_path(tar,base_name))
        make_folder(get_file_dirname(tar))
        # 复制到文件夹
        if os.path.isdir(tar):
            #print('make_folder')
            make_folder(tar)
            copy(src, tar)
        else:
        # 本地[有or没有]文件,都复制过去
            return shutil.copyfile(src, tar)
    # copy文件夹
    else:
        # 源文件不存在
        if not os.path.exists(src):
            raise Exception(f'can not find src :\n{src}')
        if os.path.isfile(tar):
            # 尝试将一个文件夹复制为文件
            raise Exception(f'can not copy folder to file.\n[{src}] -->>  [{tar}]')
        
        # 覆盖模式
        if is_cover:
            sub_file_list = get_sub_files(src)
            for sub_file in sub_file_list:
                sub_file = sub_file[len(src):]
                copy(join_path(src,sub_file), join_path(tar,sub_file),is_cover)
        # 普通模式，确保目标文件夹不存在才可以进行
        else:
            try:
                shutil.copytree(src, tar)
            except Exception:
                print(f'复制失败：[{src}]-->[{tar}]  可尝试添加参数[is_cover = True]')
                raise Exception(traceback.format_exc())
                


def get_file_base_name(path,is_postfix=True):
    '''
    获取文件名
    '''
    path = os.path.normpath(path)
    rs = os.path.basename(path)

    if is_postfix:
        return rs
    else:
        name,postfix = os.path.splitext(rs)
        return name
     
def get_file_type(path):
    '''
    获取文件类型
    '''
    path = os.path.normpath(path)
    rs = os.path.basename(path)
    name,postfix = os.path.splitext(rs)
    return postfix


def get_file_dirname(path):
    return os.path.dirname(path)

def copy_file(path,new_path):
    if not os.path.exists(get_file_dirname(new_path)):
        make_folder(get_file_dirname(new_path))
    shutil.copy(path,new_path)

def del_(floder_or_path,is_keep_folder=False):
    if os.path.isdir(floder_or_path):
        del_folder(floder_or_path,is_keep_folder)
    else:
        del_file(floder_or_path)
        
def del_file(path):
    if os.path.exists(path):  # 如果文件存在
        os.remove(path)


'''        
#####################################################################
#                             文件夹方法                             #    
#####################################################################   
'''
def rename(name,new_name):
    '''
    只能改文件夹
    '''
    try:
        if os.path.exists(name):
            get_all_permission(name)
            os.rename(name,new_name)
    except Exception:
        traceback.print_exc()

def move(src,dst):
    '''
    递归的去移动文件，它类似mv命令，其实就是重命名。
    '''
    try:
        if os.path.exists(src):
            shutil.move(src, dst)
    except Exception:
        traceback.print_exc()

def get_folder_objs(path):
    '''
    获得一个path的路径列表
    '''
    path=os.path.abspath(path)
    return path.split('\\')


def get_sub_files(path,full_path = True,include_folder=False,include='*',exclude=None,step=0):
    rs_list=[]
    if type(exclude) != list:
        exclude=[exclude]
    if type(include) != list:
        include=[include]

    for root , dirs , files in os.walk(path,topdown=True):
        for name in files:
            file_full_path = os.path.normpath(os.path.join(root,name))
            if __str_list_filter(file_full_path,exclude,include):
                if full_path:
                    rs_list.append(file_full_path)
                else:
                    rs_list.append(name)
        if not include_folder :
            continue

        for name in dirs:
            file_full_path = os.path.normpath(os.path.join(root,name))
            if __str_list_filter(file_full_path,exclude,include):
                if full_path:
                    rs_list.append(file_full_path)
                else:
                    rs_list.append(name)
        step -= 1
        if step == 0:
            return rs_list
    return rs_list

def get_the_lastest_file(path,full_path = True,include_folder=False,include='*',exclude=None,step=0):
    '''返回最新创建的文件'''
    files = get_sub_files(path,full_path,include_folder,include,exclude,step)
    if files==[]:
        return None
    lastest_file = max(files, key=lambda f: os.path.getctime(f))
    return lastest_file

def __str_list_filter(src_str,exclude_list,include_list):
    for find_str in exclude_list:
        if find_str:
            if src_str.find(find_str)>-1:
                return False
    if include_list==['*']:
        return True
    for find_str in include_list:
        if find_str:
            if src_str.find(find_str)>-1:
                return True
    return False


def clear_folder(folder_path,include_str='*',exclude=None):
    '''
    使用递归的方式,清空一个文件夹以及子文件夹,可以应用get_sub_files的过滤规则
    '''
    sub_list = get_sub_files(folder_path , include=include_str , exclude=exclude)
    for sub_file in sub_list:
        try:
            os.remove(sub_file)
        except Exception:
            traceback.print_exc()

def make_folder(folder_path):
    '''
    创建一个文件夹
    '''
    if os.path.exists(folder_path):
        return folder_path
    folder_path = os.path.abspath(folder_path)
    drive, tail = os.path.splitdrive(folder_path)
    folder_path = drive+re.sub(r'[/:*?"<>|]', '_', tail)
    try:
        os.makedirs(folder_path,mode=0o777)
    except FileExistsError:
        pass
    except BaseException:
        traceback.print_exc()
    finally:
        return folder_path
def del_folder(path,is_keep_folder=True):
    '''
    将此目录改名之后,慢慢清理,使用多线程,可以节省很多时间~~~
    '''
    global pool
    init_pool()
    if os.path.exists(path):
        temp_time=int(time.time())
        temp_path=path+str(temp_time)
        try:
            rename(path,temp_path)
            pool.submit(_del_folder_thread,temp_path)
            time.sleep(0.1)
        except BaseException:
            print(f'无法删除，未关闭{path}')
            # 执行失败，大部分原因是文件夹没关闭
            # 尝试直接删除里面文件
            files = get_sub_files(path)
            for file in files:
                del_file(file)
    if is_keep_folder:
        make_folder(path)

       
def _del_folder_thread(path):
    if os.path.exists(path):
        get_all_permission(path)
        shutil.rmtree(path)

        
def get_all_permission(path):
    if os.path.isfile(path):
        os.chmod(path, stat.S_IRWXU|stat.S_IRWXG|stat.S_IRWXO)
        return
    for f in path_tool.get_sub_files(path):
        os.chmod(f, stat.S_IRWXU|stat.S_IRWXG|stat.S_IRWXO)


    '''
    基于MD5进行对比,只按照子文件相对路劲作为key,MD5作为value
    例如 : D:\\a\\b 和 D:\\a\\new_b 进行比较。
    最后把文件提取到D:\\to_path,这个提取过程会保留目录结构
    '''



def pick_diff_file_list(src_folder_path , tar_folder_path , include_str='*' , exclude_str=None):
    '''
    基于MD5进行对比,只按照子文件相对路径作为key,MD5作为value
    例如 : D:\\a\\b 和 D:\\a\\new_b 进行比较。
    '''
    just_in_src=[]
    just_in_tar=[]
    update_list=[]

    src_list=pick_diff_file_handle(src_folder_path,include_str=include_str,exclude_str=exclude_str)
    tar_list=pick_diff_file_handle(tar_folder_path,include_str=include_str,exclude_str=exclude_str)

    diff_file_list =[]
    diff_file_list.extend(list(set(src_list).difference(set(tar_list))))
    diff_file_list.extend(list(set(tar_list).difference(set(src_list))))

    for diff_file in diff_file_list:
        if diff_file in src_list:
            just_in_src.append(diff_file)
        if diff_file in tar_list:
            just_in_tar.append(diff_file)

    same_file_list = list(set(src_list).intersection(set(tar_list)))
    for same_file in same_file_list:
        src_path = path_tool.join_path(src_folder_path,same_file)
        tar_path = path_tool.join_path(tar_folder_path,same_file)

        if file_tool.get_md5(src_path) != file_tool.get_md5(tar_path):
            update_list.append(same_file)

    return just_in_src,just_in_tar,update_list


def pick_diff_file_handle(folder_path,include_str='*',exclude_str=None):
    path_len=len(os.path.normpath(folder_path))
    rs_list=[]
    sub_list = path_tool.get_sub_files(folder_path,include_str=include_str,exclude_str=exclude_str)
    for sub_path in sub_list:
        rs_list.append(sub_path[path_len:len(sub_path)])
    return rs_list



'''        
#####################################################################
#                             其他方法                               #    
#####################################################################   
'''  
# 根目录path，一般是命令行所在的位置，也就是执行python xxxx.py的地方
def get_root_path():
    return os.getcwd()

# 启动脚本path,一般情况下，启动脚本.py会在根目录中
def get_main_path():
    return sys.path[0]

# 启动脚本name
def get_main_name():
    return sys.argv[0]

# 传入的这个方法的名字
def get_now_fuc_name(func):
    return func.__name__
def join_path(path1,*path2):
    path1 = path1 if isinstance(path1,str) else str(path1)
    if not isinstance(path2,tuple):
        path2 = [path2]
    for path_add in path2:
        path_add = path_add if isinstance(path_add,str) else str(path_add)
        if path_add.startswith('\\') or path_add.startswith('/'):
            path_add = path_add[1:len(path_add)]
        path1 = os.path.join(path1,path_add)

    return relative_2_absolute(os.path.normpath(path1))
# 当前这一句，正在执行的py文件的路径
# os.path.realpath(__file__)

# 当前这一句，正在执行的py文件的目录
# os.path.split(os.path.realpath(__file__))[0]

# 当前这一句，正在执行的py文件的名字
# os.path.split(os.path.realpath(__file__))[1]

def get_path_status(path):
    '''
    一个路径的状态,默认结尾包含[.]的是文件,虽然文件夹也可以带[.],但不这样就没法搞了\n
    1 # 存在，且是文件\n
    2 # 存在，且是文件夹\n
    3 # 不存在，且是文件\n
    4 # 不存在，且是文件夹\n
    '''
    
    if os.path.isfile(path):
        return 1
    elif os.path.isdir(path):
        return 2
    base_name = get_file_base_name(path)
    if base_name.find('.')>-1:
        return 3
    else:
        return 4

def wait_folder_prepare(path):
    '''
    等待一个文件夹完全准备好文件,复制大文件的情况可能不适用(会提前占长度，导致失败)，通常是解压缩文件的时候
    '''
    file_list = get_sub_files(path)
    for file in file_list:
        wait_file_prepare(file)

def wait_file_prepare(path,time_out=600):
    start_time = time.time()
    total = file_tool.get_file_size(path)
    while time.time() - start_time < time_out:
        time.sleep(2)
        temp = file_tool.get_file_size(path)
        if temp == total and temp > 0:
            return True
        total = temp
    return False

def is_exists(path):
    return os.path.exists(path)

def is_dir(path):
    return os.path.isdir(path)

def is_file(path):
    return os.path.isfile(path)

def relative_2_absolute(path):
    '''可以直接调用join_path'''
    return os.path.abspath(path)


import os
import shutil
import zipfile
from rogue_tools import path_tool

def zip(src_path_or_list,out_path,keep_folder=True,compression=None):
	'''
	 "zip", "tar", "gztar", "bztar",
    or "xztar"
	'''
	z = zipfile.ZipFile(out_path,'w',compression=zipfile.ZIP_LZMA)
	__zip(z,src_path_or_list,keep_folder)
	z.close()
def __zip(zip_obj:zipfile.ZipFile,file_or_folder,keep_folder=True,root_folder=''):
	# 如果是列表,遍历之后塞进去
	if type(file_or_folder) == list:
		for path_str in file_or_folder:
			__zip(zip_obj,path_str,keep_folder)
	elif path_tool.is_dir(file_or_folder):
		sub_files = path_tool.get_sub_files(file_or_folder)
		for sub_file in sub_files:
			__zip(zip_obj,sub_file,keep_folder,file_or_folder)
	else:
		if path_tool.is_exists(file_or_folder):
			if keep_folder:
				zip_obj.write(file_or_folder,file_or_folder)
			else:
				zip_obj.write(file_or_folder,os.path.basename(file_or_folder))


def unzip(src_path,out_path):
	'''
	 "zip", "tar", "gztar", "bztar",
    or "xztar"
	'''
	shutil.unpack_archive(src_path,out_path,'zip')

def is_zip_file(src_path):
	return zipfile.is_zipfile(src_path)

	try:
		# 尝试获取zip文件，可以解析就是zip
		zipfile.ZipFile(src_path, 'r')
		return True
	except Exception:
		return False



import configparser

import os
import traceback
import re


def get_ini(path,node):
    rs = None
    try:
        ini=configparser.ConfigParser()
        with open(path,"r",encoding='utf8') as f:
            lines = f.read().splitlines()
        rs = re.search('(\[)[\s\S]+',str(lines[0]))
        lines[0] = rs.group()
        ini.read_file(lines)
        #ini.sections()
        rs=dict(ini[node])

        return rs
    except BaseException as e:
        print('read ini failed',path)
        traceback.print_exc()
        return {}


def save_ini(path,node,ini_dic,is_cover_node = False):
    ini=configparser.ConfigParser()
    if os.path.exists(path):
        #ini.read(path,encoding="utf-8")
        with open(path,"r",encoding='utf8') as f:
            lines = f.read().splitlines()
        rs = re.search('(\[)[\s\S]+',str(lines[0]))
        lines[0] = rs.group()
        ini.read_file(lines)
    if is_cover_node:
        ini.remove_section(node)

    if not node in ini:
        ini.add_section(node)

    for key in ini_dic:
        #print(node,key,ini_dic[key])
        ini.set(node,key,str(ini_dic[key]))
    


    ini.write(open(path, "w",encoding="utf-8"))

def get_complete_ini(path):
    rs = {}
    try:
        ini=configparser.ConfigParser()
        with open(path,"r",encoding='utf8') as f:
            lines = f.read().splitlines()
        re_rs = re.search('(\[)[\s\S]+',str(lines[0]))
        lines[0] = re_rs.group()
        ini.read_file(lines)
        for node in ini.sections():
            rs[node]={}
            for key in ini[node]:
                rs[node][key] = str(ini[node][key])

        return rs
    except BaseException:
        traceback.print_exc()
        return rs


def save_complete_ini(path,ini_dic):
    '''
    保存完整的配置
    '''
    ini=configparser.ConfigParser()
    if os.path.exists(path):
        with open(path,"r",encoding='utf8') as f:
            lines = f.read().splitlines()
        rs = re.search('(\[)[\s\S]+',str(lines[0]))
        lines[0] = rs.group()
        ini.read_file(lines)

    for node_name in ini_dic:
        node_dic = ini_dic[node_name]
        if not node_name in ini:
                ini.add_section(node_name)
        for key_name in node_dic:
            ini.set(node_name,key_name,str(node_dic[key_name]))

    ini.write(open(path, "w",encoding="utf-8"))


import traceback
import matplotlib.pyplot as pyplot
import numpy


def __pre(title,info_dic):
    pyplot.rcParams['font.family'] = 'SimHei'
    pyplot.rcParams['axes.unicode_minus']=False
    pyplot.rcParams['figure.figsize'] = (8,3)
    pyplot.title(title)
    #pyplot.style.use('dark_background') 
    colors = pyplot.get_cmap('YlGnBu')(numpy.linspace(0.2,0.7,len(info_dic)))
    color  = '#37ACC3'
    return colors,color




def out_bar_chart(title,info_dic:dict,out_path):
    '''
    柱形图
    info_dic = {'a1':100,'b2':200,'c3':300,'d4':900}
    show_tool.out_bar_chart(info_dic,'test.png')
    '''
    x_label_list = list(info_dic.keys())
    x = numpy.arange(len(x_label_list))
    y = numpy.array(list(info_dic.values()))
    colors,color = __pre(title,info_dic)
    pyplot.bar(x, y, tick_label=x_label_list, width=0.3,color=color)
    pyplot.savefig(out_path)
    pyplot.clf()
    pyplot.show()

    

def out_cake_chart(title,info_dic:dict,out_path):
    '''
    饼图
    表现的是数据的比例
    info_dic = {'a1':100,'b2':200,'c3':300,'d4':900}
    show_tool.out_bar_chart(info_dic,'test.png')
    '''

    labels = list(info_dic.keys())
    x = numpy.array(list(info_dic.values()))
    colors,color = __pre(title,info_dic)
    # 数值，标签，顺时针，偏转90度，颜色
    pyplot.pie(x=x, labels=labels, counterclock=False, startangle=90, colors=colors)
    pyplot.savefig(out_path)
    pyplot.clf()
    #pyplot.show()



def out_line_chart(title,info_dic:dict,out_path):
    '''
    折线图
    表现的是数据变化情况
    info_dic = {'a1':100,'b2':200,'c3':300,'d4':900}
    show_tool.out_line_chart(info_dic,'test.png')
    '''
    
    labels = list(info_dic.keys())
    x = numpy.array(list(info_dic.values()))
    colors,color = __pre(title,info_dic)
    pyplot.plot(labels, x, color=color)
    pyplot.savefig(out_path)
    pyplot.clf()
    #pyplot.show()

def out_histogram(title,info_dic:dict,out_path):
    '''
    直方图
    表现的是数据的分布情况,各有多少个
    info_dic = {'a1':100,'b2':200,'c3':300,'d4':900}
    show_tool.out_line_chart(info_dic,'test.png')
    '''
    labels = list(info_dic.keys())
    x = numpy.array(list(info_dic.values()))
    colors,color = __pre(title,info_dic)
    pyplot.hist(x, color=color)
    pyplot.savefig(out_path)
    pyplot.clf()
    #pyplot.show()

def out_scatter_diagram_2D(title,info_dic:dict,out_path):
    '''
    二维散点图
    info_dic = {'a1':(100,100),'b2':(200,200),'c3':(300,300),'d4':(900,900)}
    '''
    labels = list(info_dic.keys())
    info   = list(info_dic.values())
    x_list = []
    y_list = []
    for key in info_dic:
        try:
            x_list.append(info_dic[key][0])
            y_list.append(info_dic[key][1])
            
        except Exception:
            traceback.print_exc()
            print('[Error]:out_scatter_diagram_2D,parms index error。' , key , info_dic)
            return
    x = numpy.array(x_list)
    y = numpy.array(y_list)

    colors,color = __pre(title,info_dic)
    pyplot.scatter(x, y)
    pyplot.savefig(out_path)
    pyplot.clf()
    #pyplot.show()


def out_scatter_diagram_3D(title,info_dic:dict,out_path):
    '''
    三维散点图
    info_dic = {'a1':(100,100,100),'b2':(200,200,200),'c3':(300,300,300),'d4':(900,900,900)}
    '''
    labels = list(info_dic.keys())
    info   = list(info_dic.values())
    x_list = []
    y_list = []
    z_list = []
    for key in info_dic:
        try:
            x_list.append(info_dic[key][0])
            y_list.append(info_dic[key][1])
            z_list.append(info_dic[key][2])
        except Exception:
            traceback.print_exc()
            print('[Error]:out_scatter_diagram_3D,parms index error。' , key , info_dic)
            return
    x = numpy.array(x_list)
    y = numpy.array(y_list)
    z = numpy.array(z_list)
    colors,color = __pre(title,info_dic)
    ax3d = pyplot.subplot(projection = '3d')

    ax3d.scatter(x , y , z ,c = color)
    pyplot.savefig(out_path)
    pyplot.clf()
    #pyplot.show()







import os
import socket
import requests
import re
import time
from tqdm import tqdm
from concurrent.futures import ThreadPoolExecutor
import traceback


def get_ip_public(timeout=60):
    '''公网ip'''
    text = requests.get('http://myip.ipip.net',timeout=timeout).text
    pattern = r"\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3}"
    rs = re.search(pattern,text)
    return rs.group()

def get_ip_local_area_network():
    '''局域网ip'''
    with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as s:
        s.connect(("114.114.114.114", 80))
        return s.getsockname()[0]



def download_url(url,save_folder_path,is_cover=True,is_show_process=True):
    url=url.replace('\\','/')
    if not save_folder_path.endswith('\\'):
        save_folder_path=save_folder_path+'\\'
        
    
    temp=url.split('/')
    file_name=temp[len(temp)-1]
    save_path=save_folder_path+file_name
    
    #下载条件检查,覆盖模式或者不存在文件的，才下载。
    if is_cover or not os.path.isfile(save_path):
        r = requests.get(url, stream=True)

        if is_show_process:
            total = int(r.headers.get('content-length',0))
            with open(save_path,'wb') as file,tqdm(
                desc            = file_name,
                total           = total,
                unit            = 'iB',
                unit_scale      = True,
                unit_divisor    = 1024,
                mininterval     = 2,
            )as bar:
                for data in r.iter_content(chunk_size=1024):
                    size = file.write(data)
                    bar.update(size)
        else:
            with open(save_path,'wb') as file:
                for chunk in r.iter_content(chunk_size=1024):
                    if chunk:
                        file.write(chunk)


    return save_path


def thread_download_url(url,save_folder_path,is_cover=True,time_out=180):
    '''
    建议使用这个方法下载apk,防止卡死
    '''

    pool = ThreadPoolExecutor(max_workers=3)
    try:
        future1 = pool.submit(download_url, url , save_folder_path , is_cover)
        for i in range(0,time_out):
            time.sleep(1)
            if future1.done():
                time.sleep(3)
                return i
        return -1
    finally:
        pool.shutdown()


from concurrent.futures import ThreadPoolExecutor
import concurrent.futures as futures
import traceback
import time

class Singleton(type):
    '''单例元类'''
    _instances = {}
    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super().__call__(*args, **kwargs)
        return cls._instances[cls]

class ThreadPools(ThreadPoolExecutor):
    '''多例'''
    def __init__(self,max_workers = 8) -> None:
        super().__init__(max_workers=max_workers)
        self.future_list=[]
        self.future_dic:dict[str,futures.Future]={}
        self.is_stop = False # 退出标记，线程中自行检查是否退出

        
    def set_max_workers(self, max_workers):
        self._max_workers = max_workers
        self._adjust_thread_count()

    def add_task(self,name,task,*args, **kwargs):
        '''
            # 增加一个多线程任务
            name在一轮多线程中需要唯一,便于找到自己的任务索引。\n
            name为None时,wait_finish不会等待这个任务的执行结果
        '''
        if name == None:
            self.submit(task,*args, **kwargs)
        else:
            self.future_dic[name]=self.submit(task,*args, **kwargs)
        

    def wait_finish(self, time_out=300) -> dict:
        '''等待执行结果，超过300秒就不等了'''
        rs_dic = {}
        start_time = time.time()
        while time.time() - start_time < time_out and not self.is_stop:
            for future_name, future in self.future_dic.items():
                if not future.running():
                    rs_dic[future_name] = future.result()

            if len(rs_dic) == len(self.future_dic):
                break

            time.sleep(0.05)
        self.future_dic = {}
        return rs_dic
        

    def restart(self, wait: bool = ..., *, cancel_futures: bool = ...) -> None:
        super().shutdown(wait, cancel_futures=cancel_futures)
        self.future_list=[]
        super().__init__(max_workers=self._max_workers)
        return

class ThreadPool(ThreadPools,metaclass=Singleton):
    '''单例'''
    def __init__(self,max_workers = 8) -> None:
        self.future_list=[]
        self.future_dic:dict[str,futures.Future]={}
        self.is_stop = False # 退出标记，线程中自行检查是否退出
        super().__init__(max_workers=max_workers)


import difflib
import random
# 相似度比较

def diff(str1,str2):
    seq   = difflib.SequenceMatcher(None,str1,str2)
    ratio = seq.ratio()
    return ratio

def rnd_str(length,include_lower=True,include_upper=True,include_number=True,include_symbol=False):
    add_str = ''
    if include_lower:
        add_str+='qwertyuiopasdfghjklzxcvbnm'
    if include_upper:
        add_str+='QWERTYUIOPASDFGHJKLZXCVBNM'
    if include_number:
        add_str+='1234567890'
    if include_symbol:
        add_str+='~!@#$%^&*()_+[]\;,./|:"<>?'
    return ''.join(random.sample(add_str, length))


import yaml
import re
import json
import traceback
from rogue_tools import path_tool
from loguru import logger
'''
    配合proto_tool使用，通过这个来获得【extension_name和io接口名称】
    通过这两个参数，就可以引入proto导出的xxxxx_pb2.py。
'''

class YamlObj():
    '''
    #####################################################################
    #                         一个自定义yaml对象                         #    
    #####################################################################  
    '''
    def __init__(self,file_path):
        self.file_path=file_path
        self.cmd_dic={}
        self.all_request_name = []
        self.all_reply_name = []
        try:
            yaml_obj            = yaml.safe_load(open(self.file_path, encoding='utf-8'))
            self.name           = yaml_obj.get('name')
            self.ext_id         = yaml_obj.get('extensionId')
            yaml_cmd_list       = yaml_obj.get('cmdList')
            if yaml_cmd_list:
                for cmd in yaml_cmd_list:
                    yaml_cmd = YamlCmd(cmd)
                    self.cmd_dic[yaml_cmd.index]=yaml_cmd
                    if yaml_cmd.request:
                        self.all_request_name.append(yaml_cmd.request)
                    if yaml_cmd.reply:
                        self.all_reply_name.append(yaml_cmd.reply)
        except BaseException:
            traceback.print_exc()

    def get_all_cmd(self):
        return self.cmd_dic

    def get_all_request_name(self):
        return self.all_request_name

    def get_all_reply_name(self):
        return self.all_reply_name

    def get_extension_name(self):
        return self.name
        
    def get_request_name(self,index:int):
        return self.cmd_dic[index].request

    def get_reply_name(self,index:int):
        return self.cmd_dic[index].reply

class YamlCmd():
    def __init__(self,cmd_line) -> None:

        self.rs_dic={}
        pattern = re.compile(r'[(]([0-9]+)[)]', re.I)   # re.I 表示忽略大小写
        m = pattern.search(cmd_line)
        self.index     = int(m.group()[1:-1])
        self.name      = yaml_cmd_handle(cmd_line,'void')
        self.request   = yaml_cmd_handle(cmd_line,'@in')
        self.reply     = yaml_cmd_handle(cmd_line,'@out')


def yaml_cmd_handle(src_str,start_str):
    pattern = re.compile(r'('+start_str+')([ ]+)([^,)]+)', re.I)   # re.I 表示忽略大小写
    m = pattern.search(src_str)
    if m:
        pattern2 = re.compile(r'([^,)]+)', re.I)
        s=pattern2.search(m.group()[len(start_str):])
        return s.group()
    return None




class YamlGrop():
    '''
    所有yaml的聚合类\n
    '''
    def __init__(self,folder) -> None:
        self.yaml_dic = {}
        self.name_to_extid = {}
        self.yaml_name_list=[]
        sub_list = path_tool.get_sub_files(folder,full_path=True,include='.yaml',exclude='EnumDef.yaml')
        for sub_file in sub_list:
            yaml = YamlObj(sub_file)
            self.yaml_dic[yaml.ext_id]=yaml
            self.name_to_extid[yaml.name]=yaml.ext_id
            self.yaml_name_list.append(yaml.name)

    def get_all_yaml_name_list(self):
        return self.yaml_name_list

    def get_yaml_ext_id(self,ext_id_or_name):
        if type(ext_id_or_name)==int:
            return ext_id_or_name
        return self.name_to_extid.get(ext_id_or_name,None)
    def get_yaml(self,ext_id_or_name)->YamlObj:
        ext_id = self.get_yaml_ext_id(ext_id_or_name)
        return self.yaml_dic.get(ext_id,None)

    def get_interface_info(self,ext_id_or_name,cmd_id):
        '''
        获得一个接口的信息
        '''
        yaml = self.get_yaml(ext_id_or_name)
        request_name = yaml.get_request_name(cmd_id)
        reply_name = yaml.get_reply_name(cmd_id)
        return yaml.name,request_name,reply_name




import os
import hashlib
import time
import traceback
from rogue_tools import path_tool, time_tool,file_tool


def read_simple_text(path,my_encoding='utf8'):
    rs = ['']
    with open(path,"r",encoding=my_encoding) as f:
        rs = f.read().splitlines()
    return rs

def read_simple_text2(path,my_encoding='utf8'):
    rs = None
    with open(path,"r",encoding=my_encoding) as f:
        rs = f.read(file_tool.get_file_size(path))
    return rs

def write_str(path,write_string,my_mode="a+",my_encoding='utf8'):
    if not os.path.exists(path):
        base_dir = path_tool.get_file_dirname(path)
        path_tool.make_folder(base_dir)
    with open(path,my_mode,encoding=my_encoding) as f:
        f.write(str(write_string))

def write_lines(path,write_list,mode="a+",encoding='utf8'):
    temp=""
    for line in write_list:
        temp = f'{temp}{line}\n'
        #temp=temp+line+'\n'
    write_str(path,temp,my_mode=mode,my_encoding=encoding)

def new_file(path):
    if not os.path.exists(path):
        base_dir = path_tool.get_file_dirname(path)
        path_tool.make_folder(base_dir)
        write_str(path,'')

def clear_file(path):
    write_str(path,'',my_mode='w+')

def get_file_size(path):
    return os.path.getsize(path) if os.path.exists(path) else 0

def get_folder_size(path):
    total = 0
    sub_files = path_tool.get_sub_files(path)
    for sub_file in sub_files:
        total = total + get_file_size(sub_file)
    return total

def wait_file(path,time_out=600):
    '''
    等待一个文件可用，并且不再更改。
    适用于目标文件正在创建、下载、复制的情况
    最少需要2秒,默认最大等待600秒
    '''
    start_time = time_tool.time_stamp_s()
    last_bytes = None
    file_size  = 0
    while (time_tool.time_stamp_s()-start_time) < time_out:
        fo = None
        try:
            with open(path,'rb') as fo:
                fo.seek(-10,2)
                now_bytes = fo.read(10)
                now_size = get_file_size(path)
                # 读取文件的最后几个字节,如果不同，说明还不行
                # 读取文件的大小，如果文件大小还在变化，说明还不行
                if (last_bytes == now_bytes) and (file_size == now_size):
                    # 终于等到你，还好我没放弃！！！
                    return True
                last_bytes = now_bytes
                file_size = now_size
        except (FileNotFoundError,PermissionError,):
            continue
        finally:
            time.sleep(1)
    # 待到魂归故里时，只见白首不见君
    return False

def change_1024(size):
    if size < 1024:
        return size
    if size < 1048576:
        return str(int(size/1024))+'k'
    if size < 1073741824:
        return str(int(size/1048576))+'M'
    if size < 1099511627776:
        return str(int(size/1073741824))+'G'
def read_big_file():
    pass
def get_simple_info(file_path):
    t = os.path.getatime(filePath)
    return TimeStampToTime(t)

def get_FileAccessTime(filePath):
    '''
    获取文件
    访问时间+创建时间+修改时间
    '''
    a = os.path.getatime(filePath)
    c = os.path.getctime(filePath)
    m = os.path.getmtime(filePath)
    return a,c,m



def get_md5(path):
    m = hashlib.md5()  #创建md5对象
    with open(path,'rb') as f:
        while True:
            data = f.read(40960)
            if not data:
                break
            m.update(data) #更新md5对象
    return m.hexdigest()  #返回md5对象

def txt_to_html(file,title='网页版',out_html_file = None):
    # 运行日志转成html输出
    if out_html_file == None:
        out_html_file = file+'.html'
    try:
        html_head = f'<!DOCTYPE html><html><head><meta charset="utf-8"><title>{title}</title></head><body>\n<p>'
        file_str = read_simple_text2(file)
        file_str = file_str.replace('\n','\n<p>')
        rs = f'{html_head}{file_str}'[:-3]
        write_str(out_html_file,rs,'w+')
    except BaseException:
        traceback.print_exc()
    return out_html_file


#! /usr/bin/env python
# -*- coding: utf-8 -*-
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
import setuptools
import time
from rogue_tools import path_tool,file_tool
path_tool.del_('build')
path_tool.del_('dist')
path_tool.del_('rogue_tools.egg-info')
path_tool.copy(r'C:\Users\luohao\AppData\Local\Programs\Python\Python310\Lib\site-packages\rogue_tools',r'D:\project\test_tools\rogue_tools_for_pip\rogue_tools',is_cover=True)
lines = file_tool.read_simple_text2('version.txt').split('.')
now_version = f'{lines[0]}.{lines[1]}.{int(lines[2])+1}'
file_tool.write_str('version.txt',now_version,'w+')
time.sleep(1)


setup(
    name='rogue_tools',  # 包的名字
    author='luohao',  # 作者
    version=now_version,  # 版本号
    license='MIT',

    description='private tools',  # 描述
    long_description='''long description''',
    author_email='luohao@aobi.com',  # 你的邮箱**
    url='',  # 可以写github上的地址，或者其他地址
    # 包内需要引用的文件夹
    # packages=setuptools.find_packages(exclude=['url2io',]),
    packages=["rogue_tools"],
    # keywords='NLP,tokenizing,Chinese word segementation',
    # package_dir={'jieba':'jieba'},
    # package_data={'jieba':['*.*','finalseg/*','analyse/*','posseg/*']},

    # 依赖包
    install_requires=[
        'openpyxl >= 3.0.10',
        "requests >= 2.28.1",
        "matplotlib >= 3.7.0",
        "tqdm >= 4.64.1",
        
    ],
    classifiers=[
        # 'Development Status :: 4 - Beta',
        'Operating System :: Microsoft',  # 你的操作系统  OS Independent      Microsoft
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        # 'License :: OSI Approved :: BSD License',  # BSD认证
        'Programming Language :: Python',  # 支持的语言
        'Programming Language :: Python :: 3',  # python版本 。。。
        'Programming Language :: Python :: 3.10',
        'Topic :: Software Development :: Libraries'
    ],
    zip_safe=True,
)



import traceback
import win32api 
import win32con 
import win32gui 
import win32process 
import win32clipboard
import os 
import signal 
from ctypes import *
import time
import win32com.client

from rogue_tools import  string_tool, time_tool

def get_screen_info():
    w = win32api.GetSystemMetrics(win32con.SM_CXSCREEN)
    h = win32api.GetSystemMetrics(win32con.SM_CYSCREEN)
    return (w,h)

def get_win_hwnd(exe_name):
    '''
    # 获取win进程的句柄
    '''
    return win32gui.FindWindow(None,exe_name) 


def find_win(exe_name,search_type='diff'):
    '''
    尝试用不完整名字搜索程序名
    search_type='diff',查找最接近的字符串,从而获得结果(hwnd,name)
    search_type='in'  ,查找exe_name在其中的字符串
    search_type='all' ,精准查找
    '''
    rs=(0,None)
    rate=0
    dic = get_win_dic('')
    for obj in dic:
        if search_type == 'all':
            if exe_name == dic[obj]:
                return (obj,dic[obj])

        if search_type == 'in':
            if exe_name in dic[obj]:
                return (obj,dic[obj])

        if search_type == 'diff':
            temp_rate=string_tool.diff(exe_name,dic[obj])
            if temp_rate>rate:
                rate = temp_rate
                rs=(obj,dic[obj])
    return rs 

def get_win_text(hwnd):
    '''
    # 通过句柄获取窗口名字
    '''
    return win32gui.GetWindowText(hwnd)

def get_win_hwnd_list(search_str):
    '''
    # 通过已知字符串搜索窗口
    '''
    dic = get_win_dic(search_str)
    return list(dic.keys())

def get_win_name_list(search_str):
    '''
    # 通过已知字符串搜索窗口
    '''
    dic = get_win_dic(search_str)
    return list(dic.values())

def get_win_dic(search_str):
    rs_dic={}
    def get_list_handle(hwnd,search_str):
        name = win32gui.GetWindowText(hwnd)
        if search_str in name:
            rs_dic[hwnd]=name
    win32gui.EnumChildWindows(0, get_list_handle , search_str)
    return rs_dic

def get_thread(hwnd):
    '''
    进程ID
    '''
    thread,processId =win32process.GetWindowThreadProcessId(hwnd) 
    return thread
def get_processid(hwnd):
    '''
    任务管理器中的PID
    '''
    thread,processId =win32process.GetWindowThreadProcessId(hwnd) 
    return processId
def hide_win1(hwnd):
    '''
    最小化
    '''
    print(f'hide:{hwnd}')
    win32gui.CloseWindow(hwnd)

def hide_win2(hwnd):
    '''
    隐藏,图标也看不到的那种
    '''
    print(f'hide:{hwnd}')
    win32gui.ShowWindow(hwnd, win32con.SW_HIDE) 
def show_win(hwnd):
    '''
    将窗口去掉隐藏,取消最小化,并放到最前面
    '''
    win32gui.ShowWindow(hwnd, win32con.SW_SHOWNORMAL) 
    '''
    pywin32模块下的一个小bug，在该函数调用前，需要先发送一个其他键给屏幕，如ALT键
    '''
    shell = win32com.client.Dispatch("WScript.Shell")
    shell.SendKeys('%')
    win32gui.SetForegroundWindow(hwnd)
    shell.SendKeys('%')
    
def mov_win(hwnd,x,y):
    '''
    保持窗口大小,移动到某个位置
    '''
    w,h=get_win_size(hwnd)
    set_win_pos(hwnd,x,y,w,h)
    time.sleep(0.1)

def set_win_pos(hwnd,x,y,w,h):
    '''
    激活窗口，移动到某个位置,并设置大小,通常不这么用,但挺好用的
    
    '''
    show_win(hwnd)
    win32gui.SetWindowPos(hwnd,win32con.HWND_TOPMOST,x,y,w-1,h-1,win32con.SWP_SHOWWINDOW)    
    time.sleep(0.1)

    # 如果全屏化了，可能移动失败，获取一下大小宽度，看看上一步是否移动成功
    # 成功了就说明不需要按alt+enter
    if not get_win_size(hwnd) == (w-1,h-1):
        press_union_key([VK_CODE_ALT],VK_CODE_ENTER)
        time.sleep(0.5)

    win32gui.SetWindowPos(hwnd,win32con.HWND_TOPMOST,x,y,w,h,win32con.SWP_SHOWWINDOW)    
    time.sleep(0.1)


def get_win_pos(hwnd):
    '''
    获取窗口位置
    '''
    rs = win32gui.GetWindowRect(hwnd)
    
    return (rs[0],rs[1])
def get_win_size(hwnd):
    '''
    获取窗口大小
    '''
    rs = win32gui.GetWindowRect(hwnd)
    return (rs[2]-rs[0],rs[3]-rs[1])
def kill_exe(hwnd):
    '''
    关闭程序
    '''
    if type(hwnd)==int:
        if hwnd:
            processid=get_processid(hwnd)
            try:
                os.kill(processid,signal.SIGBREAK)
                #print(f'kill-{hwnd}-{processid}')
            except BaseException:
                traceback.print_exc()
    else:
        hwnd = get_win_hwnd(hwnd)
        kill_exe(hwnd)
def loop_kill_exe(app_name):
    '''
    循环查找应用,然后杀掉
    '''
    hwnd,name = find_win(app_name,search_type='in') 
    print(f'try killing [{app_name}] , find [{hwnd},{name}]')
    if hwnd:
        thread,processId =win32process.GetWindowThreadProcessId(hwnd) 
        os.kill(processId,signal.SIGBREAK)
        time.sleep(3)
        return loop_kill_exe(app_name)
def kill_by_exe(exe_name):
    cmd = f'TASKKILL /im {exe_name}'
    os.system(cmd)
def re_start(app_name,start_path,start_time_out=60,search_type='all'):
    '''
    尝试重启,start_time_out是启动之后,等待查找句柄的最长时间
    search_type见find_win()解释
    '''
    loop_kill_exe(app_name)
        
    cmd = 'start "" '+start_path
    print(cmd)
    os.system(cmd)
    hwnd = 0
    start_time=time_tool.time_stamp_s()
    while True:
        time.sleep(5)
        until_time = time_tool.time_stamp_s() - start_time
        if until_time > start_time_out:
            return 0
        hwnd = find_win(app_name,search_type=search_type)[0]
        if hwnd:
            return hwnd
        print(f'wait:{until_time}/{start_time_out}s {hwnd}')

def get_clipboard():
    win32clipboard.OpenClipboard()
    rs = win32clipboard.GetClipboardData(win32clipboard.CF_UNICODETEXT)
    win32clipboard.CloseClipboard()
    return rs
    
# ******************************鼠标相关内容******************************
def mouse_click(pos):
    if pos==None:
        pos=get_mouse_pos()
    #x = pos[0]
    #y = pos[1]
    win32api.SetCursorPos(pos)
    mouse_exe(1)


def mouse_click_double(pos):
    if pos==None:
        pos=get_mouse_pos()
    #x = pos[0]
    #y = pos[1]
    win32api.SetCursorPos(pos)
    mouse_exe(2)
def mouse_right_click(pos):
    if pos==None:
        pos=get_mouse_pos()
    #x = pos[0]
    #y = pos[1]
    win32api.SetCursorPos(pos)
    mouse_exe(1,False)

def mouse_exe(times,right_or_left=True,intevarl=0.05):
    if right_or_left==True:
        down = win32con.MOUSEEVENTF_LEFTDOWN
        up   = win32con.MOUSEEVENTF_LEFTUP
    else:
        down = win32con.MOUSEEVENTF_RIGHTDOWN
        up   = win32con.MOUSEEVENTF_RIGHTUP

    for i in range(0,times):
        time.sleep(intevarl)
        win32api.mouse_event(down, 0, 0, 0, 0)
        time.sleep(0.02)
        win32api.mouse_event(up, 0, 0, 0, 0)
    time.sleep(0.2)


def get_mouse_pos():
    '''
    获取当前鼠标位置
    '''
    return win32gui.GetCursorPos(0)

def mouse_move(pos):
    '''
    移动鼠标位置
    '''
    win32api.SetCursorPos(pos)


def key_input(str=''):
    for c in str:
        press_key(c)
    time.sleep(0.2)  

def press_key(key='',times=1):
    # 使用VK_CODE_XXXXXX即可，其他的不一定支持喔！
    key = key.lower()
    for i in range(0,times):
        win32api.keybd_event(VK_CODE[key],0,0,0)
        win32api.keybd_event(VK_CODE[key],0,win32con.KEYEVENTF_KEYUP,0)
        time.sleep(0.1)
def press_union_key(union_list,p_key):
    '''
    按住union_list里的所有键,再按press_key
    '''
    for key in union_list:
        win32api.keybd_event(VK_CODE[key],0,0,0)
    press_key(p_key)
    for key in union_list:
        win32api.keybd_event(VK_CODE[key],0,win32con.KEYEVENTF_KEYUP,0)
    pass

    


#键盘映射
VK_CODE_BACKSPACE                   =    'backspace'
VK_CODE_TAB                         =    'tab'
VK_CODE_CLEAR                       =    'clear'
VK_CODE_ENTER                       =    'enter'
VK_CODE_SHIFT                       =    'shift'
VK_CODE_CTRL                        =    'ctrl'
VK_CODE_ALT                         =    'alt'
VK_CODE_PAUSE                       =    'pause'
VK_CODE_CAPS_LOCK                   =    'caps_lock'
VK_CODE_ESC                         =    'esc'
VK_CODE_SPACE                       =    'space'
VK_CODE_PAGE_UP                     =    'page_up'
VK_CODE_PAGE_DOWN                   =    'page_down'
VK_CODE_END                         =    'end'
VK_CODE_HOME                        =    'home'
VK_CODE_LEFT_ARROW                  =    'left_arrow'
VK_CODE_UP_ARROW                    =    'up_arrow'
VK_CODE_RIGHT_ARROW                 =    'right_arrow'
VK_CODE_DOWN_ARROW                  =    'down_arrow'
VK_CODE_SELECT                      =    'select'
VK_CODE_PRINT                       =    'print'
VK_CODE_EXECUTE                     =    'execute'
VK_CODE_PRINT_SCREEN                =    'print_screen'
VK_CODE_INSERT                      =    'insert'
VK_CODE_DEL                         =    'del'
VK_CODE_HELP                        =    'help'
VK_CODE_0                           =    '0'
VK_CODE_1                           =    '1'
VK_CODE_2                           =    '2'
VK_CODE_3                           =    '3'
VK_CODE_4                           =    '4'
VK_CODE_5                           =    '5'
VK_CODE_6                           =    '6'
VK_CODE_7                           =    '7'
VK_CODE_8                           =    '8'
VK_CODE_9                           =    '9'
VK_CODE_A                           =    'a'
VK_CODE_B                           =    'b'
VK_CODE_C                           =    'c'
VK_CODE_D                           =    'd'
VK_CODE_E                           =    'e'
VK_CODE_F                           =    'f'
VK_CODE_G                           =    'g'
VK_CODE_H                           =    'h'
VK_CODE_I                           =    'i'
VK_CODE_J                           =    'j'
VK_CODE_K                           =    'k'
VK_CODE_L                           =    'l'
VK_CODE_M                           =    'm'
VK_CODE_N                           =    'n'
VK_CODE_O                           =    'o'
VK_CODE_P                           =    'p'
VK_CODE_Q                           =    'q'
VK_CODE_R                           =    'r'
VK_CODE_S                           =    's'
VK_CODE_T                           =    't'
VK_CODE_U                           =    'u'
VK_CODE_V                           =    'v'
VK_CODE_W                           =    'w'
VK_CODE_X                           =    'x'
VK_CODE_Y                           =    'y'
VK_CODE_Z                           =    'z'
VK_CODE_NUMPAD_0                    =    'numpad_0'
VK_CODE_NUMPAD_1                    =    'numpad_1'
VK_CODE_NUMPAD_2                    =    'numpad_2'
VK_CODE_NUMPAD_3                    =    'numpad_3'
VK_CODE_NUMPAD_4                    =    'numpad_4'
VK_CODE_NUMPAD_5                    =    'numpad_5'
VK_CODE_NUMPAD_6                    =    'numpad_6'
VK_CODE_NUMPAD_7                    =    'numpad_7'
VK_CODE_NUMPAD_8                    =    'numpad_8'
VK_CODE_NUMPAD_9                    =    'numpad_9'
VK_CODE_MULTIPLY_KEY                =    'multiply_key'
VK_CODE_ADD_KEY                     =    'add_key'
VK_CODE_SEPARATOR_KEY               =    'separator_key'
VK_CODE_SUBTRACT_KEY                =    'subtract_key'
VK_CODE_DECIMAL_KEY                 =    'decimal_key'
VK_CODE_DIVIDE_KEY                  =    'divide_key'
VK_CODE_F1                          =    'f1'
VK_CODE_F2                          =    'f2'
VK_CODE_F3                          =    'f3'
VK_CODE_F4                          =    'f4'
VK_CODE_F5                          =    'f5'
VK_CODE_F6                          =    'f6'
VK_CODE_F7                          =    'f7'
VK_CODE_F8                          =    'f8'
VK_CODE_F9                          =    'f9'
VK_CODE_F10                         =    'f10'
VK_CODE_F11                         =    'f11'
VK_CODE_F12                         =    'f12'
VK_CODE_F13                         =    'f13'
VK_CODE_F14                         =    'f14'
VK_CODE_F15                         =    'f15'
VK_CODE_F16                         =    'f16'
VK_CODE_F17                         =    'f17'
VK_CODE_F18                         =    'f18'
VK_CODE_F19                         =    'f19'
VK_CODE_F20                         =    'f20'
VK_CODE_F21                         =    'f21'
VK_CODE_F22                         =    'f22'
VK_CODE_F23                         =    'f23'
VK_CODE_F24                         =    'f24'
VK_CODE_NUM_LOCK                    =    'num_lock'
VK_CODE_SCROLL_LOCK                 =    'scroll_lock'
VK_CODE_LEFT_SHIFT                  =    'left_shift'
VK_CODE_RIGHT_SHIFT                 =    'right_shift'
VK_CODE_LEFT_CONTROL                =    'left_control'
VK_CODE_RIGHT_CONTROL               =    'right_control'
VK_CODE_LEFT_MENU                   =    'left_menu'
VK_CODE_RIGHT_MENU                  =    'right_menu'
VK_CODE_BROWSER_BACK                =    'browser_back'
VK_CODE_BROWSER_FORWARD             =    'browser_forward'
VK_CODE_BROWSER_REFRESH             =    'browser_refresh'
VK_CODE_BROWSER_STOP                =    'browser_stop'
VK_CODE_BROWSER_SEARCH              =    'browser_search'
VK_CODE_BROWSER_FAVORITES           =    'browser_favorites'
VK_CODE_BROWSER_START_AND_HOME      =    'browser_start_and_home'
VK_CODE_VOLUME_MUTE                 =    'volume_mute'
VK_CODE_VOLUME_DOWN                 =    'volume_Down'
VK_CODE_VOLUME_UP                   =    'volume_up'
VK_CODE_NEXT_TRACK                  =    'next_track'
VK_CODE_PREVIOUS_TRACK              =    'previous_track'
VK_CODE_STOP_MEDIA                  =    'stop_media'
VK_CODE_PLAY_PAUSE_MEDIA            =    'play\pause_media'
VK_CODE_START_MAIL                  =    'start_mail'
VK_CODE_SELECT_MEDIA                =    'select_media'
VK_CODE_START_APPLICATION_1         =    'start_application_1'
VK_CODE_START_APPLICATION_2         =    'start_application_2'
VK_CODE_ATTN_KEY                    =    'attn_key'
VK_CODE_CRSEL_KEY                   =    'crsel_key'
VK_CODE_EXSEL_KEY                   =    'exsel_key'
VK_CODE_PLAY_KEY                    =    'play_key'
VK_CODE_ZOOM_KEY                    =    'zoom_key'
VK_CODE_CLEAR_KEY                   =    'clear_key'
VK_CODE_PLUS                        =    '+'
VK_CODE_COMMA                       =    ','
VK_CODE_MINUS                       =    '-'
VK_CODE_DOT                         =    '.'
VK_CODE_SLASH                       =    '/'
VK_CODE_DOT2                        =    '`'
VK_CODE_SEMICOLON                   =    ';'
VK_CODE_LEFT_MID_BRACKETS           =    '['
VK_CODE_SLASH2                      =    '\\'#这是一个斜杠
VK_CODE_RIGHT_MID_BRACKETS          =    ']'
VK_CODE_DOT3                        =    "'"#这是一个'


VK_CODE = {
VK_CODE_BACKSPACE:0x08,
VK_CODE_TAB:0x09,
VK_CODE_CLEAR:0x0C,
VK_CODE_ENTER:0x0D,
VK_CODE_SHIFT:0x10,
VK_CODE_CTRL:0x11,
VK_CODE_ALT:0x12,
VK_CODE_PAUSE:0x13,
VK_CODE_CAPS_LOCK:0x14,
VK_CODE_ESC:0x1B,
VK_CODE_SPACE:0x20,
VK_CODE_PAGE_UP:0x21,
VK_CODE_PAGE_DOWN:0x22,
VK_CODE_END:0x23,
VK_CODE_HOME:0x24,
VK_CODE_LEFT_ARROW:0x25,
VK_CODE_UP_ARROW:0x26,
VK_CODE_RIGHT_ARROW:0x27,
VK_CODE_DOWN_ARROW:0x28,
VK_CODE_SELECT:0x29,
VK_CODE_PRINT:0x2A,
VK_CODE_EXECUTE:0x2B,
VK_CODE_PRINT_SCREEN:0x2C,
VK_CODE_INSERT:0x2D,
VK_CODE_DEL:0x2E,
VK_CODE_HELP:0x2F,
VK_CODE_0:0x30,
VK_CODE_1:0x31,
VK_CODE_2:0x32,
VK_CODE_3:0x33,
VK_CODE_4:0x34,
VK_CODE_5:0x35,
VK_CODE_6:0x36,
VK_CODE_7:0x37,
VK_CODE_8:0x38,
VK_CODE_9:0x39,
VK_CODE_A:0x41,
VK_CODE_B:0x42,
VK_CODE_C:0x43,
VK_CODE_D:0x44,
VK_CODE_E:0x45,
VK_CODE_F:0x46,
VK_CODE_G:0x47,
VK_CODE_H:0x48,
VK_CODE_I:0x49,
VK_CODE_J:0x4A,
VK_CODE_K:0x4B,
VK_CODE_L:0x4C,
VK_CODE_M:0x4D,
VK_CODE_N:0x4E,
VK_CODE_O:0x4F,
VK_CODE_P:0x50,
VK_CODE_Q:0x51,
VK_CODE_R:0x52,
VK_CODE_S:0x53,
VK_CODE_T:0x54,
VK_CODE_U:0x55,
VK_CODE_V:0x56,
VK_CODE_W:0x57,
VK_CODE_X:0x58,
VK_CODE_Y:0x59,
VK_CODE_Z:0x5A,
VK_CODE_NUMPAD_0:0x60,
VK_CODE_NUMPAD_1:0x61,
VK_CODE_NUMPAD_2:0x62,
VK_CODE_NUMPAD_3:0x63,
VK_CODE_NUMPAD_4:0x64,
VK_CODE_NUMPAD_5:0x65,
VK_CODE_NUMPAD_6:0x66,
VK_CODE_NUMPAD_7:0x67,
VK_CODE_NUMPAD_8:0x68,
VK_CODE_NUMPAD_9:0x69,
VK_CODE_MULTIPLY_KEY:0x6A,
VK_CODE_ADD_KEY:0x6B,
VK_CODE_SEPARATOR_KEY:0x6C,
VK_CODE_SUBTRACT_KEY:0x6D,
VK_CODE_DECIMAL_KEY:0x6E,
VK_CODE_DIVIDE_KEY:0x6F,
VK_CODE_F1:0x70,
VK_CODE_F2:0x71,
VK_CODE_F3:0x72,
VK_CODE_F4:0x73,
VK_CODE_F5:0x74,
VK_CODE_F6:0x75,
VK_CODE_F7:0x76,
VK_CODE_F8:0x77,
VK_CODE_F9:0x78,
VK_CODE_F10:0x79,
VK_CODE_F11:0x7A,
VK_CODE_F12:0x7B,
VK_CODE_F13:0x7C,
VK_CODE_F14:0x7D,
VK_CODE_F15:0x7E,
VK_CODE_F16:0x7F,
VK_CODE_F17:0x80,
VK_CODE_F18:0x81,
VK_CODE_F19:0x82,
VK_CODE_F20:0x83,
VK_CODE_F21:0x84,
VK_CODE_F22:0x85,
VK_CODE_F23:0x86,
VK_CODE_F24:0x87,
VK_CODE_NUM_LOCK:0x90,
VK_CODE_SCROLL_LOCK:0x91,
VK_CODE_LEFT_SHIFT:0xA0,
VK_CODE_RIGHT_SHIFT:0xA1,
VK_CODE_LEFT_CONTROL:0xA2,
VK_CODE_RIGHT_CONTROL:0xA3,
VK_CODE_LEFT_MENU:0xA4,
VK_CODE_RIGHT_MENU:0xA5,
VK_CODE_BROWSER_BACK:0xA6,
VK_CODE_BROWSER_FORWARD:0xA7,
VK_CODE_BROWSER_REFRESH:0xA8,
VK_CODE_BROWSER_STOP:0xA9,
VK_CODE_BROWSER_SEARCH:0xAA,
VK_CODE_BROWSER_FAVORITES:0xAB,
VK_CODE_BROWSER_START_AND_HOME:0xAC,
VK_CODE_VOLUME_MUTE:0xAD,
VK_CODE_VOLUME_DOWN:0xAE,
VK_CODE_VOLUME_UP:0xAF,
VK_CODE_NEXT_TRACK:0xB0,
VK_CODE_PREVIOUS_TRACK:0xB1,
VK_CODE_STOP_MEDIA:0xB2,
VK_CODE_PLAY_PAUSE_MEDIA:0xB3,
VK_CODE_START_MAIL:0xB4,
VK_CODE_SELECT_MEDIA:0xB5,
VK_CODE_START_APPLICATION_1:0xB6,
VK_CODE_START_APPLICATION_2:0xB7,
VK_CODE_ATTN_KEY:0xF6,
VK_CODE_CRSEL_KEY:0xF7,
VK_CODE_EXSEL_KEY:0xF8,
VK_CODE_PLAY_KEY:0xFA,
VK_CODE_ZOOM_KEY:0xFB,
VK_CODE_CLEAR_KEY:0xFE,
VK_CODE_PLUS:0xBB,
VK_CODE_COMMA:0xBC,
VK_CODE_MINUS:0xBD,
VK_CODE_DOT:0xBE,
VK_CODE_SLASH:0xBF,
VK_CODE_DOT2:0xC0,
VK_CODE_SEMICOLON:0xBA,
VK_CODE_LEFT_MID_BRACKETS:0xDB,
VK_CODE_SLASH2:0xDC,
VK_CODE_RIGHT_MID_BRACKETS:0xDD,
VK_CODE_DOT3:0xDE
}


import os
import sys
import time
import traceback

from PyQt5 import QtCore
from PyQt5.QtWidgets import *
from rogue_tools import file_tool,path_tool,thread_tool


class mainUI():
    def __init__(self,w,h,title='PyQt5 - UI by [rogue_tools]',is_center=True):
        self.config_path        = 'save_params.txt'
        self.save_dic           = self.load_input()
        self.editor_dic         = {}
        self.ui_width           = w
        self.ui_height          = h
        self.pool               = thread_tool.ThreadPool()
        self.app                = QApplication(sys.argv)
        self.windows            = QWidget()
        self.windows.resize(self.ui_width,self.ui_height)
        self.windows.setWindowTitle(title)
        # 居中
        if is_center:
            qr = self.windows.frameGeometry()
            cp = QDesktopWidget().availableGeometry().center()
            qr.moveCenter(cp)
            self.windows.move(qr.topLeft())


    def show(self):
        #show()方法在屏幕上显示出widget组件
        self.windows.show()
        #循环执行窗口触发事件，结束后不留垃圾的退出，不添加的话新建的widget组件就会一闪而过
        exe_code = self.app.exec_()
        print(exe_code)
        self.exit_exe()
        sys.exit(exe_code)
    
    def add_label(self,line_index,title, start_pos = (0,0),obj_w=200,obj_h=20):
        label = QLabel(self.windows)
        label.setAlignment(QtCore.Qt.AlignTop | QtCore.Qt.AlignLeft)
        label.setGeometry(QtCore.QRect(start_pos[0] , obj_h * line_index+start_pos[1] , obj_w , obj_h))
        label.setText(title)

    def add_btn(self,line_index,title, start_pos = (0,0),obj_w=100,obj_h=20,call_func=None,in_thread=False,func_parms=[]):
        '''竖着放按钮'''
        #设置按钮并给按钮命名
        btn = self._add_btn(title,start_pos[0] , obj_h * line_index+start_pos[1] , obj_w , obj_h,call_func,in_thread,func_parms)
        return btn

    def add_btn_horizontal(self,line_index,title, start_pos = (0,0),obj_w=100,obj_h=20,call_func=None,in_thread=False, func_parms=[]):
        '''横着放按钮'''
        #设置按钮并给按钮命名
        btn = self._add_btn(title,obj_w * line_index+start_pos[0],start_pos[1] , obj_w , obj_h,call_func,in_thread,func_parms)
        return btn

    def _add_btn(self,title,x,y,w,h,call_func,in_thread,func_parms):
        '''谨慎使用in_thread,通常界面元素无法在多线程中操作,一般用于无界面的情况'''
        def run():
            if in_thread:
                self.pool.add_task(title,call_func,*func_parms)
            else:
                call_func(*func_parms)
        btn = QPushButton(title,self.windows)
        btn.setGeometry(x,y,w,h)
        if call_func:
            btn.clicked.connect(run)
        return btn

    def add_input_editor(self,line_index,title, start_pos = (0,0),obj_w_1=70,obj_w_2=200,obj_h = 20 ,edit_text='',edit_tips=''):
        input_str = self.save_dic.get(title,'') if edit_text=='' else edit_text
        # 显示标签
        label = QLabel(self.windows)
        label.setGeometry(QtCore.QRect(start_pos[0] , obj_h * line_index+start_pos[1] , obj_w_1 , obj_h))
        label.setText(title)
        # 输入框
        edit = QLineEdit(self.windows)
        edit.setPlaceholderText(str(edit_tips))
        edit.setText(str(input_str))
        edit.setGeometry(QtCore.QRect(obj_w_1+start_pos[0] , obj_h * line_index+start_pos[1] , obj_w_2 , obj_h))
        # 管理内容
        self.editor_dic[title] = edit.text
        return edit
    
    def add_combo_box(self,line_index,title, start_pos = (0,0),obj_w_1=70,obj_w_2=200,obj_h = 20,item_list=[]):
        currentText = self.save_dic.get(title,'')
        # 显示标签
        label = QLabel(self.windows)
        label.setGeometry(QtCore.QRect(start_pos[0] , obj_h * line_index+start_pos[1] , obj_w_1 , obj_h))
        label.setText(title)
        # 下拉框
        comb_box = QComboBox(self.windows)
        comb_box.addItems(item_list)
        comb_box.setGeometry(QtCore.QRect(start_pos[0]+obj_w_1 , obj_h * line_index+start_pos[1] , obj_w_2 , obj_h))
        comb_box.setCurrentText(currentText)
        self.editor_dic[title]=comb_box.currentText
        return comb_box

    def add_scrollableTextEdit(self,line_index,title, start_pos = (0,0),obj_w=500,obj_h = 200):
        ed = QTextEdit(self.windows)
        ed.setGeometry(QtCore.QRect(QtCore.QRect(start_pos[0] , obj_h * line_index+start_pos[1] , obj_w , obj_h)))
        #currentText = self.save_dic.get(title,'')
        #ed.setPlainText(currentText)
        ed.verticalScrollBar().setValue(ed.verticalScrollBar().maximum())
        return ed

    def scrollableTextEdit_append(self,ed:QTextEdit,text):
        lines = [] if ed.toPlainText()=='' else ed.toPlainText().split('\n')
        lines.append(f'{str(len(lines)).ljust(3)}:'+text)
        new_text = '\n'.join(lines)
        ed.setPlainText(new_text)
        # 滚动到文本框的底部，以显示最新行
        ed.verticalScrollBar().setValue(ed.verticalScrollBar().maximum())

    def exit_exe(self):
        self.pool.is_stop=True
        self.pool.shutdown(wait=False)
        QtCore.QCoreApplication.instance().quit()

    def msgbox(self,msg_str,title=''):
        QMessageBox.about(self.windows, title,msg_str)
        
    def question_box(self,msg_str,title=''):
        reply = QMessageBox.question(self.windows,title,msg_str)
        return reply == QMessageBox.Yes


    def save_input(self):
        write_lines=[]
        for key in self.editor_dic:
            write_lines.append(f'{key}={self.editor_dic[key]()}')
        print(f'save:{write_lines}')
        file_tool.write_lines(self.config_path,write_lines,'w+')

    def load_input(self):
        rs_dic = {}
        if not path_tool.is_exists(self.config_path):
            return rs_dic
        lines = file_tool.read_simple_text(self.config_path)
        
        for line in lines:
            temp  = line.split('=')
            if len(temp)==2:
                rs_dic[temp[0]]=temp[1]
        return rs_dic

if __name__ == '__main__':
    from rogue_tools import string_tool
    def test(k):
        # 示范多线程
        key =f'{string_tool.rnd_str(5), k, input.text(), box.currentText()}'
        print(f'3秒后输出{key}')
        time.sleep(3)
        print(key)

    ui = mainUI(300,600)
    start_pos = (10,10)
    btn1      = ui.add_btn(0,'竖着放1',start_pos, call_func= test, func_parms=['竖着放1111'])
    btn2      = ui.add_btn(1,'竖着放2',start_pos, call_func= test, func_parms=['竖着放2222'])
    start_pos = (10,60)
    btn3      = ui.add_btn_horizontal(0,'横着放1',start_pos, call_func= test, func_parms=['横着放1111'])
    btn4      = ui.add_btn_horizontal(1,'横着放2',start_pos, call_func= test, func_parms=['横着放2222'])
    start_pos = (10,110)
    input     = ui.add_input_editor(0,'input0',start_pos,edit_text='删掉显示提示',edit_tips='提示')
    box       = ui.add_combo_box(1,'box0',start_pos,item_list=['box_str_1','box_str_2','box_str_3'])
    label     = ui.add_label(2,'label0',start_pos)
    ui.show()


import time
import re
import os
import sys
import subprocess
import traceback
from rogue_tools import android_tool,time_tool,thread_tool,path_tool
from loguru import logger



def add_root_path_for_airtest(src_path):
    '''
    传入启动脚本的位置,一般情况下是 : src_path = os.path.realpath(__file__)
    '''
    try:
        root_path=os.path.join(os.path.split(src_path)[0], "..", "..", "..", "..", "..")
        if not root_path in sys.path:
            sys.path.append(root_path)
    except BaseException:
        traceback.print_exc()

class Android():
    def __init__(self,adb_path,device_name,device_brand) -> None:
        self.device_name = device_name
        self.adb_path = adb_path
        self.device_brand = device_brand

    def adb_execute(self,cmd):
        adb = f'{self.adb_path} -s {self.device_name}  {cmd}'
        logger.debug(adb)
        result = subprocess.run(adb, capture_output=True, text=True, shell=True)
        return result.stdout.split('\n')


class DevicesManager(metaclass=thread_tool.Singleton):
    def __init__(self) -> None:
        self.adb_path = 'adb'
        self.devices:dict[Android] = {}
        self.devices_name = {}
        self.tp = thread_tool.ThreadPool()
        #self.init_device_dic(60)

    def __del__(self):
        self.tp.shutdown()

    def set_adb(self,adb_exe):
        '''设置用于命令行的adb执行器'''
        self.adb_path = adb_exe

    def get_all_connected_devices(self):
        result = subprocess.run(f'{self.adb_path} devices', capture_output=True, text=True, shell=True)
        # 提取设备名称，忽略空行
        devices_name = [
            re.match(r'(\S+)\s+device', line).group(1) 
            for line in result.stdout.split('\n')[1:]
            if re.match(r'(\S+)\s+device', line)
        ]
        return devices_name

    def connect(self, device_dic:dict,try_times=15):
        '''连接设备'''
        
        for device in device_dic:
            try:
                subprocess.run(f'{self.adb_path} connect {device}', capture_output=True, text=True, shell=True)
            except BaseException:
                logger.warning(f'connect {device} failed')

        for _ in range(try_times):
            # 运行 adb devices 命令，获取连接的设备信息
            self.devices_name = self.get_all_connected_devices()
            logger.debug(f'connecting:{self.devices_name}')

            # 创建设备对象字典
            self.devices = {}
            for device in self.devices_name:
                device_brand =  device_dic.get(device,None)
                if device_brand:
                    self.devices[device] = Android(self.adb_path, device,device_brand) 
            print(self.devices)
            # 如果找到设备，停止循环
            if self.devices:
                logger.debug(f'find {self.devices}')
                self.device_name_list = [device for device in self.devices]
                print(self.device_name_list)
                return
            # 等待一秒再尝试
            time.sleep(1)



    def kill_adb_server(self):
        adb = f'{self.adb_path} kill-server'
        return subprocess.run(adb, capture_output=True, text=True, shell=True)

    def get_device_name_list(self):
        return self.device_name_list
    
    def run_in_all(self, adb_cmd, device_name=None,in_thread=True):
        '''用于执行命令,多设备会自动转为多线程模式'''
        loop_dic = {device_name: self.devices.get(device_name)} if device_name else self.devices
        if in_thread:
            rs_dic = {device_name: result for device_name, result in self.tp.map(
                lambda d: (d[0], d[1].adb_execute(adb_cmd)),
                loop_dic.items()
            )}
        else:
            rs_dic = {device_name: loop_dic[device_name].adb_execute(adb_cmd) for device_name in loop_dic}
        print(rs_dic,loop_dic)
        return rs_dic

    def get_apk_info(self, device_name=None):
        '''获得当前处于活动状态的apk的包名和主活动名字,需要先将app置于活动状态'''
        rs_dic = {}
        dumpsys_window_cmd = 'shell dumpsys window'
        window_info_dic = self.run_in_all(dumpsys_window_cmd, device_name)

        # 用于匹配主活动名的正则表达式
        def extract_apk_info(line):
            activity_match = re.search(r'mFocusedWindow=\S+ \S+ (\S+/\S+)}', line)
            if activity_match:
                package_name, activity_name = activity_match.group(1).split('/')
                return package_name, activity_name
            return None, None

        for device_name,window_info_list in window_info_dic.items():
            rs_dic[device_name]=next((extract_apk_info(line) for line in window_info_list if extract_apk_info(line) != (None, None)), (None, None))
        return rs_dic


    def get_installed_apk(self, device_name=None):
        '''获取已安装的apk列表'''
        install_dic = self.run_in_all('shell pm list packages', device_name)
        # 使用列表推导式来提取不带前缀的包名，并将其放入一个新的列表中
        for device_name, package_list in install_dic.items():
            package_list = [line[8:] for line in package_list if line.startswith('package:')]
            install_dic[device_name] = package_list
        return install_dic

    def is_all_install_apk(self, apk_name):
        '''检查已连接的所有设备,是否都安装了某个apk'''
        is_install = True
        for device in self.devices:
            is_install &= self.is_install_apk(device.device_name)

    def is_install_apk(self, apk_name, device_name):
        '''检查是否安装了某个apk'''
        return apk_name in (self.get_installed_apk(device_name).get(device_name,[]))
    
    def install_apk(self,local_path,device_name=None):
        '''安装apk'''
        install_apk_cmd = f'install -g {local_path}'
        return self.run_in_all(install_apk_cmd,device_name,in_thread=False)

    def uninstall_apk(self,package_name,device_name=None):
        '''卸载apk'''
        return self.run_in_all(f'uninstall {package_name}',device_name)

    def start_app(self,package_name,activity_name,device_name=None):
        '''启动app'''
        # startActivity
        return self.run_in_all(f'shell am start -n {package_name}/{activity_name}',device_name)
        
    def stop_app(self,package_name,device_name=None):
        '''停止app'''
        return self.run_in_all(f'shell am force-stop {package_name}',device_name)

    def clear_data(self,package_name,device_name=None):
        '''清理一个apk的数据'''
        return self.run_in_all(f'shell pm clear {package_name}',device_name)

    def delete_file(self,path,device_name=None):
        '''删除文件'''
        if path in ('/','/sdcard','/storage','/system'):
            raise Exception(f'rm error,for path = {path}')
        return self.run_in_all(f'shell rm -rf {path}',device_name)
    
    def tar_folder(self,src_folder,target_full_path:str,device_name=None):
        '''将某个path打包'''
        #adb shell tar -cvf /sdcard/cache.tar /sdcard/cache
        target_full_path = target_full_path if target_full_path.endswith('.tar') else target_full_path + '.tar'
        self.run_in_all(f'shell tar -cvf {target_full_path} {src_folder}',device_name)
        self._wait_file_prepare(target_full_path,device_name)
        return True

    def pull_file(self,file_or_files,local_folder,device_name=None):
        '''
        拉取安卓文件,可能会报权限错误,这个无解
        一般先tar打包,再拉
        '''
        #adb pull /sdcard/Android/data/com.baitian.spacex.sx.bt/files/CustomAstralLog.log E:\\test
        if type(file_or_files)==list:
            for file in file_or_files:
                self.pull_file(file,local_folder)
        else:
            target_local_folder = path_tool.join_path(local_folder,device_name)
            target_local_folder = path_tool.make_folder(target_local_folder)
            self.run_in_all(f'pull {file_or_files} {target_local_folder}',device_name)
            # 要等待，就返回等待的结果
            base_name = path_tool.get_file_base_name(file_or_files)
            path_tool.wait_file_prepare(path_tool.join_path(target_local_folder,base_name))


    def get_file_size(self,file,device_name=None):
        #adb shell stat -c "%s"
        if file.endswith('/'):
            print(f'Warning!Your path [ {file} ] is folder,not file.Then you can see 3488')
        rs_dic = {}
        try:
            temp_dic = self.run_in_all(f'shell stat -c "%s" {file}',device_name)
            for key,values in temp_dic.items():
                for size in values:
                    if len(size)>0:
                        rs_dic[device_name]=int(size)
        except BaseException:
            traceback.print_exc()
        return rs_dic

    def _wait_file_prepare(self,file,device_name=None,time_out = 600):
        last_size = 0
        is_continue = True
        start_time = time.time()
        while is_continue:
            if time.time() - start_time > time_out:
                break
            time.sleep(3)
            is_continue = True
            size_dic = self.get_file_size(file,device_name)
            for device_name,size in size_dic.items():
                is_continue = is_continue and (size == last_size and size>0) # size变化
                print(f'waiting file:{int(time.time() - start_time)}s , {int(size)}b')
                last_size = size
        return False


    